"""Declares :class:`AuthorizationEndpointTest`."""
from .base import BaseOpenAuthorizationTest


class AuthorizationEndpointTest(BaseOpenAuthorizationTest):
    __module__: str = 'cbra.ext.oauth2.tests'
    pass
